unit MSGSnatchers;

// $Id: MSGSnatchers.pas,v 1.5 2002/03/20 15:38:53 takao Exp $

{$ObjExportAll On}

interface

uses
  Classes, Controls, Messages, Windows;

type
  { Event Types for TMessageSnatcher }
  TMessageHandleEvent = procedure(
    aSender: TObject; var aMessage: TMessage; var aHandled: Boolean) of Object;

  { TCustomMessageSnatcher }
  TCustomMessageSnatcher = class(TComponent)
  private
    FControl: TControl;
    FOldWndProc: TWndMethod;

    FOnMessage: TMessageHandleEvent;

  protected
    { TComponent }
    procedure Notification(aComponent: TComponent; anOperation: TOperation);
      override;
    procedure Loaded; override;

    { Template Methods }
    procedure InitializeControl(aControl: TControl); virtual;
    procedure FinalizeControl(aControl: TControl); virtual;

    procedure SetControl(aControl: TControl);
    procedure MessageHandler(var aMessage: TMessage; var aHandled: Boolean);
      virtual;
    procedure WndProc(var aMessage: TMessage); virtual;

    property OldWndProc: TWndMethod read FOldWndProc;
    property Control: TControl read FControl write SetControl;

    property OnMessage: TMessageHandleEvent read FOnMessage write FOnMessage;
  public
    destructor Destroy; override;
  end;

  { TCustomWinMessageSnatcher }
  TCustomWinMessageSnatcher = class(TCustomMessageSnatcher)
  private
    hwndInitialized_: Boolean;
  protected
    function GetWinControl: TWinControl;
    procedure SetWinControl(aWinControl: TWinControl);
    procedure DoInitializeHWND;
    procedure DoFinalizeHWND;
    { TCustomMessageSnatcher }
    procedure InitializeControl(aControl: TControl); override;
    procedure FinalizeControl(aControl: TControl); override;
    procedure WndProc(var aMessage: TMessage); override;
    { Template Methods }
    procedure InitializeHWND(aHandle: HWND); virtual;
    procedure FinalizeHWND(aHandle: HWND); virtual;

    property WinControl: TWinControl read GetWinControl write SetWinControl;
  end;

  { TMessageSnatcher }
  TMessageSnatcher = class(TCustomMessageSnatcher)
  published
    property Control;
    property OnMessage;
  end;

  { Event types for TWinMessageSnatcher }
  THWNDEvent = procedure(aSender: TObject; aHandle: HWND) of object;

  { TWinMessageSnatcher }
  TWinMessageSnatcher = class(TCustomWinMessageSnatcher)
  private
    FOnInitializeHWND: THWNDEvent;
    FOnFinalizeHWND: THWNDEvent;
  protected
    { TCustomWinMessageSnatcher }
    procedure InitializeHWND(aHandle: HWND); override;
    procedure FinalizeHWND(aHandle: HWND); override;
  published
    property WinControl;
    property OnInitializeHWND: THWNDEvent
      read FOnInitializeHWND write FOnInitializeHWND;
    property OnFinalizeHWND: THWNDEvent
      read FOnFinalizeHWND write FOnFinalizeHWND;
  end;


implementation

{ TCustomMessageSnatcher }

{ TODO -oNAKAGUCHI Takao : `F[쐬A
Cӂ̃R|[lg폜Ăɓ삷悤ɂB }
{ DONE -oNAKAGUCHI Takao : vO}[eXg𓱓B }
{ DONE -oNAKAGUCHI Takao : TWinControl pMessageSnatcher쐬B
InitHWND, UninitHWND ev[g\bhBhwndValid_ tOB }

destructor TCustomMessageSnatcher.Destroy;
begin
  Control := Nil;
  inherited;
end;

procedure TCustomMessageSnatcher.FinalizeControl(aControl: TControl);
begin
end;

procedure TCustomMessageSnatcher.InitializeControl(aControl: TControl);
begin
end;

procedure TCustomMessageSnatcher.Loaded;
begin
  inherited;

  if csDesigning in ComponentState then Exit;

  if FControl <> Nil then begin
    FOldWndProc := FControl.WindowProc;
    FControl.WindowProc := WndProc;
    InitializeControl(FControl);
  end;
end;

procedure TCustomMessageSnatcher.MessageHandler(var aMessage: TMessage;
  var aHandled: Boolean);
begin
end;

procedure TCustomMessageSnatcher.Notification(aComponent: TComponent;
  anOperation: TOperation);
begin
  inherited;

  if (aComponent = FControl) and (anOperation = opRemove) then
    Control := Nil;
end;

procedure TCustomMessageSnatcher.SetControl(aControl: TControl);
begin
  if (csLoading in ComponentState) or (csDesigning in ComponentState) then begin
    FControl := aControl;
    Exit;
  end;

  if FControl <> Nil then begin
    FControl.WindowProc := FOldWndProc;
    FOldWndProc := Nil;
    FinalizeControl(FControl);
  end;

  FControl := aControl;

  if FControl <> Nil then begin
    InitializeControl(FControl);
    FOldWndProc := FControl.WindowProc;
    FControl.WindowProc := WndProc;
  end;
end;

procedure TCustomMessageSnatcher.WndProc(var aMessage: TMessage);
var
  handled: Boolean;
begin
  handled := False;

  MessageHandler(aMessage, handled);

  if (not handled) and Assigned(FOnMessage) then
    FOnMessage(Self, aMessage, handled);

  if (not handled) and Assigned(FOldWndProc) then
    FOldWndProc(aMessage);
end;


{ TCustomWinMessageSnatcher }

procedure TCustomWinMessageSnatcher.DoFinalizeHWND;
begin
  if hwndInitialized_ then begin
    FinalizeHWnd(WinControl.Handle);
    hwndInitialized_ := False;
  end;
end;

procedure TCustomWinMessageSnatcher.DoInitializeHWND;
begin
  if not hwndInitialized_ then begin
    InitializeHWnd(WinControl.Handle);
    hwndInitialized_ := True;
  end;
end;

procedure TCustomWinMessageSnatcher.FinalizeControl(aControl: TControl);
begin
  DoFinalizeHWND;
end;

procedure TCustomWinMessageSnatcher.FinalizeHWND(aHandle: HWND);
begin
end;

function TCustomWinMessageSnatcher.GetWinControl: TWinControl;
begin
  Result := Control as TWinControl;
end;

procedure TCustomWinMessageSnatcher.InitializeControl(aControl: TControl);
begin
  DoInitializeHWND;
end;

procedure TCustomWinMessageSnatcher.InitializeHWND(aHandle: HWND);
begin
end;

procedure TCustomWinMessageSnatcher.SetWinControl(aWinControl: TWinControl);
begin
  Control := aWinControl;
end;

procedure TCustomWinMessageSnatcher.WndProc(var aMessage: TMessage);
begin
  case aMessage.Msg of
    WM_CREATE:  DoInitializeHWND;
    WM_DESTROY: DoFinalizeHWND;
  end;
  inherited;
end;


{ TWinMessageSnatcher }

procedure TWinMessageSnatcher.FinalizeHWND(aHandle: HWND);
begin
  inherited;
  if Assigned(FOnFinalizeHWND) then FOnFinalizeHWND(Self, aHandle);
end;

procedure TWinMessageSnatcher.InitializeHWND(aHandle: HWND);
begin
  inherited;
  if Assigned(FOnInitializeHWND) then FOnInitializeHWND(Self, aHandle);
end;

end.
